@@ -17,7 +17,6 @@
 #define ANNOYLIB_H
 
 #include <stdio.h>
-#include <string>
 #include <sys/stat.h>
 #ifndef _MSC_VER
 #include <unistd.h>
@@ -27,6 +26,7 @@
 #include <sys/types.h>
 #include <fcntl.h>
 #include <stddef.h>
+
 #if defined(_MSC_VER) && _MSC_VER == 1500
 typedef unsigned char     uint8_t;
 typedef signed __int32    int32_t;
@@ -35,7 +35,7 @@ typedef unsigned __int64  uint64_t;
 #include <stdint.h>
 #endif
 
-#ifdef __MINGW32__
+#if defined(_MSC_VER) || defined(__MINGW32__)
  #ifndef NOMINMAX
   #define NOMINMAX
  #endif
@@ -45,6 +45,7 @@ typedef unsigned __int64  uint64_t;
  #include <sys/mman.h>
 #endif
 
+#include <cerrno>
 #include <string.h>
 #include <math.h>
 #include <vector>
@@ -68,20 +69,22 @@ typedef unsigned __int64  uint64_t;
 
 #ifndef _MSC_VER
 #define popcount __builtin_popcountll
-#elif _MSC_VER == 1500
+#else // See #293, #358
 #define isnan(x) _isnan(x)
 #define popcount cole_popcount
-#else
-#define popcount __popcnt64
 #endif
 
-#ifndef NO_MANUAL_VECTORIZATION
-#if defined(__AVX__) && defined (__SSE__) && defined(__SSE2__) && defined(__SSE3__)
+#if !defined(NO_MANUAL_VECTORIZATION) && defined(__GNUC__) && (__GNUC__ >6) && defined(__AVX512F__)  // See #402
+#pragma message "Using 512-bit AVX instructions"
+#define USE_AVX512
+#elif !defined(NO_MANUAL_VECTORIZATION) && defined(__AVX__) && defined (__SSE__) && defined(__SSE2__) && defined(__SSE3__)
+#pragma message "Using 128-bit AVX instructions"
 #define USE_AVX
-#endif
+#else
+#pragma message "Using no AVX instructions"
 #endif
 
-#ifdef USE_AVX
+#if defined(USE_AVX) || defined(USE_AVX512)
 #if defined(_MSC_VER)
 #include <intrin.h>
 #elif defined(__GNUC__)
@@ -100,11 +103,24 @@ typedef unsigned __int64  uint64_t;
 
 
 using std::vector;
-using std::string;
 using std::pair;
 using std::numeric_limits;
 using std::make_pair;
 
+inline void* remap_memory(void* _ptr, int _fd, size_t old_size, size_t new_size) {
+#ifdef __linux__
+  _ptr = mremap(_ptr, old_size, new_size, MREMAP_MAYMOVE);
+#else
+  munmap(_ptr, old_size);
+#ifdef MAP_POPULATE
+  _ptr = mmap(_ptr, new_size, PROT_READ | PROT_WRITE, MAP_SHARED | MAP_POPULATE, _fd, 0);
+#else
+  _ptr = mmap(_ptr, new_size, PROT_READ | PROT_WRITE, MAP_SHARED, _fd, 0);
+#endif
+#endif
+  return _ptr;
+}
+
 namespace {
 
 template<typename S, typename Node>
@@ -227,7 +243,81 @@ inline float euclidean_distance<float>(const float* x, const float* y, int f) {
 
 #endif
 
- 
+#ifdef USE_AVX512
+template<>
+inline float dot<float>(const float* x, const float *y, int f) {
+  float result = 0;
+  if (f > 15) {
+    __m512 d = _mm512_setzero_ps();
+    for (; f > 15; f -= 16) {
+      //AVX512F includes FMA
+      d = _mm512_fmadd_ps(_mm512_loadu_ps(x), _mm512_loadu_ps(y), d);
+      x += 16;
+      y += 16;
+    }
+    // Sum all floats in dot register.
+    result += _mm512_reduce_add_ps(d);
+  }
+  // Don't forget the remaining values.
+  for (; f > 0; f--) {
+    result += *x * *y;
+    x++;
+    y++;
+  }
+  return result;
+}
+
+template<>
+inline float manhattan_distance<float>(const float* x, const float* y, int f) {
+  float result = 0;
+  int i = f;
+  if (f > 15) {
+    __m512 manhattan = _mm512_setzero_ps();
+    for (; i > 15; i -= 16) {
+      const __m512 x_minus_y = _mm512_sub_ps(_mm512_loadu_ps(x), _mm512_loadu_ps(y));
+      manhattan = _mm512_add_ps(manhattan, _mm512_abs_ps(x_minus_y));
+      x += 16;
+      y += 16;
+    }
+    // Sum all floats in manhattan register.
+    result = _mm512_reduce_add_ps(manhattan);
+  }
+  // Don't forget the remaining values.
+  for (; i > 0; i--) {
+    result += fabsf(*x - *y);
+    x++;
+    y++;
+  }
+  return result;
+}
+
+template<>
+inline float euclidean_distance<float>(const float* x, const float* y, int f) {
+  float result=0;
+  if (f > 15) {
+    __m512 d = _mm512_setzero_ps();
+    for (; f > 15; f -= 16) {
+      const __m512 diff = _mm512_sub_ps(_mm512_loadu_ps(x), _mm512_loadu_ps(y));
+      d = _mm512_fmadd_ps(diff, diff, d);
+      x += 16;
+      y += 16;
+    }
+    // Sum all floats in dot register.
+    result = _mm512_reduce_add_ps(d);
+  }
+  // Don't forget the remaining values.
+  for (; f > 0; f--) {
+    float tmp = *x - *y;
+    result += tmp * tmp;
+    x++;
+    y++;
+  }
+  return result;
+}
+
+#endif
+
+
 template<typename T>
 inline T get_norm(T* v, int f) {
   return sqrt(dot(v, v, f));
@@ -239,7 +329,7 @@ inline void two_means(const vector<Node*>& nodes, int f, Random& random, bool co
     This algorithm is a huge heuristic. Empirically it works really well, but I
     can't motivate it well. The basic idea is to keep two centroids and assign
     points to either one of them. We weight each centroid by the number of points
-    assigned to it, so to balance it. 
+    assigned to it, so to balance it.
   */
   static int iteration_steps = 200;
   size_t count = nodes.size();
@@ -356,14 +446,12 @@ struct Angular : Base {
   }
   template<typename S, typename T, typename Random>
   static inline void create_split(const vector<Node<S, T>*>& nodes, int f, size_t s, Random& random, Node<S, T>* n) {
-    Node<S, T>* p = (Node<S, T>*)malloc(s); // TODO: avoid
-    Node<S, T>* q = (Node<S, T>*)malloc(s); // TODO: avoid
+    Node<S, T>* p = (Node<S, T>*)alloca(s);
+    Node<S, T>* q = (Node<S, T>*)alloca(s);
     two_means<T, Random, Angular, Node<S, T> >(nodes, f, random, true, p, q);
     for (int z = 0; z < f; z++)
       n->v[z] = p->v[z] - q->v[z];
     Base::normalize<T, Node<S, T> >(n, f);
-    free(p);
-    free(q);
   }
   template<typename T>
   static inline T normalized_distance(T distance) {
@@ -429,17 +517,15 @@ struct DotProduct : Angular {
 
   template<typename S, typename T, typename Random>
   static inline void create_split(const vector<Node<S, T>*>& nodes, int f, size_t s, Random& random, Node<S, T>* n) {
-    Node<S, T>* p = (Node<S, T>*)malloc(s); // TODO: avoid
-    Node<S, T>* q = (Node<S, T>*)malloc(s); // TODO: avoid
-    DotProduct::zero_value(p); 
+    Node<S, T>* p = (Node<S, T>*)alloca(s);
+    Node<S, T>* q = (Node<S, T>*)alloca(s);
+    DotProduct::zero_value(p);
     DotProduct::zero_value(q);
     two_means<T, Random, DotProduct, Node<S, T> >(nodes, f, random, true, p, q);
     for (int z = 0; z < f; z++)
       n->v[z] = p->v[z] - q->v[z];
     n->dot_factor = p->dot_factor - q->dot_factor;
     DotProduct::normalize<T, Node<S, T> >(n, f);
-    free(p);
-    free(q);
   }
 
   template<typename T, typename Node>
@@ -578,7 +664,7 @@ struct Hamming : Base {
       for (; j < dim; j++) {
         n->v[0] = j;
         cur_size = 0;
-	for (typename vector<Node<S, T>*>::const_iterator it = nodes.begin(); it != nodes.end(); ++it) {
+        for (typename vector<Node<S, T>*>::const_iterator it = nodes.begin(); it != nodes.end(); ++it) {
           if (margin(n, (*it)->v, f)) {
             cur_size++;
           }
@@ -638,12 +724,12 @@ struct Minkowski : Base {
 struct Euclidean : Minkowski {
   template<typename S, typename T>
   static inline T distance(const Node<S, T>* x, const Node<S, T>* y, int f) {
-    return euclidean_distance(x->v, y->v, f);    
+    return euclidean_distance(x->v, y->v, f);
   }
   template<typename S, typename T, typename Random>
   static inline void create_split(const vector<Node<S, T>*>& nodes, int f, size_t s, Random& random, Node<S, T>* n) {
-    Node<S, T>* p = (Node<S, T>*)malloc(s); // TODO: avoid
-    Node<S, T>* q = (Node<S, T>*)malloc(s); // TODO: avoid
+    Node<S, T>* p = (Node<S, T>*)alloca(s);
+    Node<S, T>* q = (Node<S, T>*)alloca(s);
     two_means<T, Random, Euclidean, Node<S, T> >(nodes, f, random, false, p, q);
 
     for (int z = 0; z < f; z++)
@@ -652,8 +738,6 @@ struct Euclidean : Minkowski {
     n->a = 0.0;
     for (int z = 0; z < f; z++)
       n->a += -n->v[z] * (p->v[z] + q->v[z]) / 2;
-    free(p);
-    free(q);
   }
   template<typename T>
   static inline T normalized_distance(T distance) {
@@ -675,8 +759,8 @@ struct Manhattan : Minkowski {
   }
   template<typename S, typename T, typename Random>
   static inline void create_split(const vector<Node<S, T>*>& nodes, int f, size_t s, Random& random, Node<S, T>* n) {
-    Node<S, T>* p = (Node<S, T>*)malloc(s); // TODO: avoid
-    Node<S, T>* q = (Node<S, T>*)malloc(s); // TODO: avoid
+    Node<S, T>* p = (Node<S, T>*)alloca(s);
+    Node<S, T>* q = (Node<S, T>*)alloca(s);
     two_means<T, Random, Manhattan, Node<S, T> >(nodes, f, random, false, p, q);
 
     for (int z = 0; z < f; z++)
@@ -685,8 +769,6 @@ struct Manhattan : Minkowski {
     n->a = 0.0;
     for (int z = 0; z < f; z++)
       n->a += -n->v[z] * (p->v[z] + q->v[z]) / 2;
-    free(p);
-    free(q);
   }
   template<typename T>
   static inline T normalized_distance(T distance) {
@@ -704,19 +786,21 @@ template<typename S, typename T>
 class AnnoyIndexInterface {
  public:
   virtual ~AnnoyIndexInterface() {};
-  virtual void add_item(S item, const T* w) = 0;
-  virtual void build(int q) = 0;
-  virtual void unbuild() = 0;
-  virtual bool save(const char* filename, bool prefault=false) = 0;
+  virtual bool add_item(S item, const T* w, char** error=NULL) = 0;
+  virtual bool build(int q, char** error=NULL) = 0;
+  virtual bool unbuild(char** error=NULL) = 0;
+  virtual bool save(const char* filename, bool prefault=false, char** error=NULL) = 0;
   virtual void unload() = 0;
-  virtual bool load(const char* filename, bool prefault=false) = 0;
+  virtual bool load(const char* filename, bool prefault=false, char** error=NULL) = 0;
   virtual T get_distance(S i, S j) const = 0;
   virtual void get_nns_by_item(S item, size_t n, size_t search_k, vector<S>* result, vector<T>* distances) const = 0;
   virtual void get_nns_by_vector(const T* w, size_t n, size_t search_k, vector<S>* result, vector<T>* distances) const = 0;
   virtual S get_n_items() const = 0;
+  virtual S get_n_trees() const = 0;
   virtual void verbose(bool v) = 0;
   virtual void get_item(S item, T* v) const = 0;
   virtual void set_seed(int q) = 0;
+  virtual bool on_disk_build(const char* filename, char** error=NULL) = 0;
 };
 
 template<typename S, typename T, typename Distance, typename Random>
@@ -745,11 +829,14 @@ protected:
   bool _loaded;
   bool _verbose;
   int _fd;
+  bool _on_disk;
+  bool _built;
 public:
 
-  AnnoyIndex(int f) : _f(f), _random() {
+   AnnoyIndex(int f) : _f(f), _random() {
     _s = offsetof(Node, v) + _f * sizeof(T); // Size of each node
     _verbose = false;
+    _built = false;
     _K = (S) (((size_t) (_s - offsetof(Node, children))) / sizeof(S)); // Max number of descendants to fit into node
     reinitialize(); // Reset everything
   }
@@ -761,12 +848,17 @@ public:
     return _f;
   }
 
-  void add_item(S item, const T* w) {
-    add_item_impl(item, w);
+  bool add_item(S item, const T* w, char** error=NULL) {
+    return add_item_impl(item, w, error);
   }
 
   template<typename W>
-  void add_item_impl(S item, const W& w) {
+  bool add_item_impl(S item, const W& w, char** error=NULL) {
+    if (_loaded) {
+      showUpdate("You can't add an item to a loaded index\n");
+      if (error) *error = (char *)"You can't add an item to a loaded index";
+      return false;
+    }
     _allocate_size(item + 1);
     Node* n = _get(item);
 
@@ -783,13 +875,44 @@ public:
 
     if (item >= _n_items)
       _n_items = item + 1;
+
+    return true;
   }
 
-  void build(int q) {
+  bool on_disk_build(const char* file, char** error=NULL) {
+    _on_disk = true;
+    _fd = open(file, O_RDWR | O_CREAT | O_TRUNC, (int) 0600);
+    if (_fd == -1) {
+      showUpdate("Error: file descriptor is -1\n");
+      if (error) *error = strerror(errno);
+      _fd = 0;
+      return false;
+    }
+    _nodes_size = 1;
+    if (ftruncate(_fd, _s * _nodes_size) == -1) {
+      showUpdate("Error truncating file: %s\n", strerror(errno));
+      if (error) *error = strerror(errno);
+      return false;
+    }
+#ifdef MAP_POPULATE
+    _nodes = (Node*) mmap(0, _s * _nodes_size, PROT_READ | PROT_WRITE, MAP_SHARED | MAP_POPULATE, _fd, 0);
+#else
+    _nodes = (Node*) mmap(0, _s * _nodes_size, PROT_READ | PROT_WRITE, MAP_SHARED, _fd, 0);
+#endif
+    return true;
+  }
+
+  bool build(int q, char** error=NULL) {
     if (_loaded) {
-      // TODO: throw exception
       showUpdate("You can't build a loaded index\n");
-      return;
+      if (error) *error = (char *)"You can't build a loaded index";
+      return false;
+    }
+
+    if (_built) {
+      showUpdate("You can't build a built index\n");
+      if (error) *error = (char *)"You can't build a built index";
+      return false;
     }
 
     D::template preprocess<T, S, Node>(_nodes, _s, _n_items, _f);
@@ -804,7 +927,7 @@ public:
 
       vector<S> indices;
       for (S i = 0; i < _n_items; i++) {
-	      if (_get(i)->n_descendants >= 1) // Issue #223
+        if (_get(i)->n_descendants >= 1) // Issue #223
           indices.push_back(i);
       }
 
@@ -819,28 +942,71 @@ public:
     _n_nodes += _roots.size();
 
     if (_verbose) showUpdate("has %d nodes\n", _n_nodes);
+
+    if (_on_disk) {
+      _nodes = remap_memory(_nodes, _fd, _s * _nodes_size, _s * _n_nodes);
+      if (ftruncate(_fd, _s * _n_nodes)) {
+	// TODO: this probably creates an index in a corrupt state... not sure what to do
+	showUpdate("Error truncating file: %s\n", strerror(errno));
+	if (error) *error = strerror(errno);
+	return false;
+      }
+      _nodes_size = _n_nodes;
+    }
+    _built = true;
+    return true;
   }
-  
-  void unbuild() {
+
+  bool unbuild(char** error=NULL) {
     if (_loaded) {
       showUpdate("You can't unbuild a loaded index\n");
-      return;
+      if (error) *error = (char *)"You can't unbuild a loaded index";
+      return false;
     }
 
     _roots.clear();
     _n_nodes = _n_items;
+    _built = false;
+
+    return true;
   }
 
-  bool save(const char* filename, bool prefault=false) {
-    FILE *f = fopen(filename, "wb");
-    if (f == NULL)
+  bool save(const char* filename, bool prefault=false, char** error=NULL) {
+    if (!_built) {
+      showUpdate("You can't save an index that hasn't been built\n");
+      if (error) *error = (char *)"You can't save an index that hasn't been built";
       return false;
+    }
+    if (_on_disk) {
+      return true;
+    } else {
+      // Delete file if it already exists (See issue #335)
+      unlink(filename);
 
-    fwrite(_nodes, _s, _n_nodes, f);
-    fclose(f);
+      //printf("path: %s\n", filename);
 
-    unload();
-    return load(filename, prefault=false);
+      FILE *f = fopen(filename, "wb");
+      if (f == NULL) {
+        showUpdate("Unable to open: %s\n", strerror(errno));
+        if (error) *error = strerror(errno);
+        return false;
+      }
+
+      if (fwrite(_nodes, _s, _n_nodes, f) != (size_t) _n_nodes) {
+        showUpdate("Unable to write: %s\n", strerror(errno));
+        if (error) *error = strerror(errno);
+        return false;
+      }
+
+      if (fclose(f) == EOF) {
+        showUpdate("Unable to close: %s\n", strerror(errno));
+        if (error) *error = strerror(errno);
+        return false;
+      }
+
+      unload();
+      return load(filename, prefault, error);
+    }
   }
 
   void reinitialize() {
@@ -850,39 +1016,61 @@ public:
     _n_items = 0;
     _n_nodes = 0;
     _nodes_size = 0;
+    _on_disk = false;
     _roots.clear();
   }
 
   void unload() {
-    if (_fd) {
-      // we have mmapped data
+    if (_on_disk && _fd) {
       close(_fd);
-      off_t size = _n_nodes * _s;
-      munmap(_nodes, size);
-    } else if (_nodes) {
-      // We have heap allocated data
-      free(_nodes);
+      munmap(_nodes, _s * _nodes_size);
+    } else {
+      if (_fd) {
+        // we have mmapped data
+        close(_fd);
+        munmap(_nodes, _n_nodes * _s);
+      } else if (_nodes) {
+        // We have heap allocated data
+        free(_nodes);
+      }
     }
     reinitialize();
     if (_verbose) showUpdate("unloaded\n");
   }
 
-  bool load(const char* filename, bool prefault=false) {
+  bool load(const char* filename, bool prefault=false, char** error=NULL) {
     _fd = open(filename, O_RDONLY, (int)0400);
     if (_fd == -1) {
+      showUpdate("Error: file descriptor is -1\n");
+      if (error) *error = strerror(errno);
       _fd = 0;
       return false;
     }
     off_t size = lseek(_fd, 0, SEEK_END);
-#ifdef MAP_POPULATE
-    const int populate = prefault ? MAP_POPULATE : 0;
-    _nodes = (Node*)mmap(
-        0, size, PROT_READ, MAP_SHARED | populate, _fd, 0);
-#else
-    _nodes = (Node*)mmap(
-        0, size, PROT_READ, MAP_SHARED, _fd, 0);
-#endif
+    if (size == -1) {
+      showUpdate("lseek returned -1\n");
+      if (error) *error = strerror(errno);
+      return false;
+    } else if (size == 0) {
+      showUpdate("Size of file is zero\n");
+      if (error) *error = (char *)"Size of file is zero";
+      return false;
+    } else if (size % _s) {
+      // Something is fishy with this index!
+      showUpdate("Error: index size %zu is not a multiple of vector size %zu\n", (size_t)size, _s);
+      if (error) *error = (char *)"Index size is not a multiple of vector size";
+      return false;
+    }
 
+    int flags = MAP_SHARED;
+    if (prefault) {
+#ifdef MAP_POPULATE
+      flags |= MAP_POPULATE;
+#else
+      showUpdate("prefault is set to true, but MAP_POPULATE is not defined on this platform");
+#endif
+    }
+    _nodes = (Node*)mmap(0, size, PROT_READ, flags, _fd, 0);
     _n_nodes = (S)(size / _s);
 
     // Find the roots by scanning the end of the file and taking the nodes with most descendants
@@ -901,6 +1089,7 @@ public:
     if (_roots.size() > 1 && _get(_roots.front())->children[0] == _get(_roots.back())->children[0])
       _roots.pop_back();
     _loaded = true;
+    _built = true;
     _n_items = m;
     if (_verbose) showUpdate("found %lu roots with degree %d\n", _roots.size(), m);
     return true;
@@ -911,6 +1100,7 @@ public:
   }
 
   void get_nns_by_item(S item, size_t n, size_t search_k, vector<S>* result, vector<T>* distances) const {
+    // TODO: handle OOB
     const Node* m = _get(item);
     _get_all_nns(m->v, n, search_k, result, distances);
   }
@@ -918,14 +1108,21 @@ public:
   void get_nns_by_vector(const T* w, size_t n, size_t search_k, vector<S>* result, vector<T>* distances) const {
     _get_all_nns(w, n, search_k, result, distances);
   }
+
   S get_n_items() const {
     return _n_items;
   }
+
+  S get_n_trees() const {
+    return _roots.size();
+  }
+
   void verbose(bool v) {
     _verbose = v;
   }
 
   void get_item(S item, T* v) const {
+    // TODO: handle OOB
     Node* m = _get(item);
     memcpy(v, m->v, (_f) * sizeof(T));
   }
@@ -938,12 +1135,20 @@ protected:
   void _allocate_size(S n) {
     if (n > _nodes_size) {
       const double reallocation_factor = 1.3;
-      S new_nodes_size = std::max(n,
-				  (S)((_nodes_size + 1) * reallocation_factor));
-      if (_verbose) showUpdate("Reallocating to %d nodes\n", new_nodes_size);
-      _nodes = realloc(_nodes, _s * new_nodes_size);
-      memset((char *)_nodes + (_nodes_size * _s)/sizeof(char), 0, (new_nodes_size - _nodes_size) * _s);
+      S new_nodes_size = std::max(n, (S) ((_nodes_size + 1) * reallocation_factor));
+      void *old = _nodes;
+
+      if (_on_disk) {
+        int rc = ftruncate(_fd, _s * new_nodes_size);
+        if (_verbose && rc) showUpdate("File truncation error\n");
+        _nodes = remap_memory(_nodes, _fd, _s * _nodes_size, _s * new_nodes_size);
+      } else {
+        _nodes = realloc(_nodes, _s * new_nodes_size);
+        memset((char *) _nodes + (_nodes_size * _s) / sizeof(char), 0, (new_nodes_size - _nodes_size) * _s);
+      }
+
       _nodes_size = new_nodes_size;
+      if (_verbose) showUpdate("Reallocating to %d nodes: old_address=%p, new_address=%p\n", new_nodes_size, old, _nodes);
     }
   }
 
@@ -984,7 +1189,7 @@ protected:
     }
 
     vector<S> children_indices[2];
-    Node* m = (Node*)malloc(_s); // TODO: avoid
+    Node* m = (Node*)alloca(_s);
     D::create_split(children, _f, _s, _random, m);
 
     for (size_t i = 0; i < indices.size(); i++) {
@@ -1031,13 +1236,12 @@ protected:
     _allocate_size(_n_nodes + 1);
     S item = _n_nodes++;
     memcpy(_get(item), m, _s);
-    free(m);
 
     return item;
   }
 
   void _get_all_nns(const T* v, size_t n, size_t search_k, vector<S>* result, vector<T>* distances) const {
-    Node* v_node = (Node *)malloc(_s); // TODO: avoid
+    Node* v_node = (Node *)alloca(_s);
     D::template zero_value<Node>(v_node);
     memcpy(v_node->v, v, sizeof(T) * _f);
     D::init_node(v_node, _f);
@@ -1073,7 +1277,7 @@ protected:
 
     // Get distances for all items
     // To avoid calculating distance multiple times for any items, sort by id
-    sort(nns.begin(), nns.end());
+    std::sort(nns.begin(), nns.end());
     vector<pair<T, S> > nns_dist;
     S last = -1;
     for (size_t i = 0; i < nns.size(); i++) {
@@ -1093,7 +1297,6 @@ protected:
         distances->push_back(D::normalized_distance(nns_dist[i].first));
       result->push_back(nns_dist[i].second);
     }
-    free(v_node);
   }
 };
 
